import { Server } from 'socket.io'
import { appLogger } from '../utilities/logger'
import { getRootPath } from '../utilities/storage'
import { readJsonValue, setJsonValue } from '../utilities/json'

class DateManager {
  private static readonly GAME_SPEED = 8 // 24 game hours / 3 real hours
  private static readonly UPDATE_INTERVAL = 1000 // 1 second

  private io: Server | null = null
  private intervalId: NodeJS.Timeout | null = null
  private currentDate: Date = new Date()

  public async boot(io: Server): Promise<void> {
    this.io = io
    await this.loadDate()
    this.startDateLoop()
    appLogger.info('Date manager loaded')
  }

  // When a GM sets the time, update the current date and update the world file
  public async setTime(time: string): Promise<void> {
    try {
      let newDate: Date;

      // Check if it's just a time (HH:mm or HH:mm:ss format)
      if (/^\d{1,2}:\d{2}(:\d{2})?$/.test(time)) {
        const [hours, minutes] = time.split(':').map(Number);
        newDate = new Date(this.currentDate); // Clone current date
        newDate.setHours(hours, minutes);
      } else {
        // Treat as full datetime string
        newDate = new Date(time);
        if (isNaN(newDate.getTime())) return;
      }

      this.currentDate = newDate;
      this.emitDate();
      await this.saveDate();
    } catch (error) {
      appLogger.error(`Failed to set time: ${error instanceof Error ? error.message : String(error)}`);
      throw error;
    }
  }

  private async loadDate(): Promise<void> {
    try {
      const dateString = await readJsonValue<string>(this.getWorldFilePath(), 'date')
      this.currentDate = new Date(dateString)
    } catch (error) {
      appLogger.error(`Failed to load date: ${error instanceof Error ? error.message : String(error)}`)
      this.currentDate = new Date() // Use current date as fallback
    }
  }

  private startDateLoop(): void {
    this.intervalId = setInterval(() => {
      this.advanceGameTime()
      this.emitDate()
      this.saveDate()
    }, DateManager.UPDATE_INTERVAL)
  }

  private advanceGameTime(): void {
    const advanceMilliseconds = DateManager.GAME_SPEED * DateManager.UPDATE_INTERVAL
    this.currentDate = new Date(this.currentDate.getTime() + advanceMilliseconds)
  }

  private emitDate(): void {
    this.io?.emit('date', this.currentDate)
  }

  private async saveDate(): Promise<void> {
    try {
      await setJsonValue(this.getWorldFilePath(), 'date', this.currentDate)
    } catch (error) {
      appLogger.error(`Failed to save date: ${error instanceof Error ? error.message : String(error)}`)
    }
  }

  private getWorldFilePath(): string {
    return getRootPath('data', 'world.json')
  }
}

export default new DateManager()
